require "import"
import "android.content.Context"
import "android.os.Vibrator"
import "android.widget.*"
import "android.view.Gravity"
import "com.androlua.*"
import "android.content.Intent"

context = activity or service
vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)

function MostrarMenu()
  -- Opciones para la interfaz
  local tipos = {"Audio", "Video"}
  local formatos_audio = {"mp3", "wav", "aac", "m4a", "opus", "vorbis"}
  local formatos_video = {"mp4", "mkv", "webm", "avi"}
  local calidades_audio = {"320", "256", "192", "128", "96"}
  local calidades_video = {"1080p", "720p", "480p", "360p", "best"}
  local plataformas = {"YouTube", "Spotify", "Otras plataformas"}
  
  local layout = {
    LinearLayout,
    orientation = "vertical",
    padding = "16dp",
    {
      TextView,
      text = "Descargador Universal con yt-dlp",
      textSize = "20sp",
      gravity = "center",
      padding = "8dp",
    },
    {
      TextView,
      text = "Selecciona plataforma:",
      textSize = "16sp",
      padding = "8dp",
    },
    {
      Spinner,
      id = "SpinnerPlatform",
      layout_width = "match_parent",
      layout_height = "wrap_content",
    },
    {
      EditText,
      id = "EntradaURL",
      hint = "Pega la URL aquí:",
      layout_width = "match_parent",
      layout_height = "wrap_content",
    },
    {
      TextView,
      text = "Tipo de contenido:",
      textSize = "16sp",
      padding = "8dp",
    },
    {
      Spinner,
      id = "SpinnerType",
      layout_width = "match_parent",
      layout_height = "wrap_content",
    },
    {
      TextView,
      text = "Formato:",
      textSize = "16sp",
      padding = "8dp",
    },
    {
      Spinner,
      id = "SpinnerFormat",
      layout_width = "match_parent",
      layout_height = "wrap_content",
    },
    {
      TextView,
      text = "Calidad:",
      textSize = "16sp",
      padding = "8dp",
    },
    {
      Spinner,
      id = "SpinnerQuality",
      layout_width = "match_parent",
      layout_height = "wrap_content",
    },
    {
      CheckBox,
      id = "CheckBoxPlaylist",
      text = "Descargar lista completa",
      checked = false,
    },
    {
      Button,
      text = "Generar Código Python",
      onClick = function()
        local URL = tostring(EntradaURL.getText())
        if URL == "" then
          vibrator.vibrate(100)
          Toast.makeText(context, "No has ingresado una URL", Toast.LENGTH_SHORT).show()
          return
        end
        
        local selectedPlatform = plataformas[SpinnerPlatform.getSelectedItemPosition() + 1]
        local selectedType = tipos[SpinnerType.getSelectedItemPosition() + 1]
        local selectedFormat, selectedQuality
        local isPlaylist = CheckBoxPlaylist.isChecked()
        
        if selectedType == "Audio" then
          selectedFormat = formatos_audio[SpinnerFormat.getSelectedItemPosition() + 1]
          selectedQuality = calidades_audio[SpinnerQuality.getSelectedItemPosition() + 1]
        else
          selectedFormat = formatos_video[SpinnerFormat.getSelectedItemPosition() + 1]
          selectedQuality = calidades_video[SpinnerQuality.getSelectedItemPosition() + 1]
        end
        
        local codigoPython = ""
        if selectedPlatform == "Spotify" then
          -- Plantilla corregida para Spotify
          local plantillaSpotify = [[
#!/usr/bin/env python3
import spotipy
from spotipy.oauth2 import SpotifyClientCredentials
from yt_dlp import YoutubeDL
from mutagen.id3 import ID3, TIT2, TPE1, TALB, APIC
import requests
from gtts import gTTS
import time
import os
import tempfile
import subprocess

CLIENT_ID = '79220c1558cb410c9bdb102f77d67447'
CLIENT_SECRET = '3b9aa57d9c7143728611a584889abab8'

def speak(texto):
    """Función para dar retroalimentación por voz con gTTS acelerada"""
    try:
        with tempfile.NamedTemporaryFile(suffix='.mp3', delete=False) as temp_file:
            temp_path = temp_file.name
        
        tts = gTTS(text=texto, lang="es")
        tts.save(temp_path)
        
        # controlar la velocidad y el volumen del tts
        os.system(f"mpv --speed=1.3 --volume=100 {temp_path}")
        
        os.remove(temp_path)
    except Exception as e:
        print(f"Error en la síntesis de voz: {str(e)}")

def actualizar_biblioteca():
    carpeta_salida = "/storage/emulated/0/Download/"
    speak("Actualizando la base de datos multimedia de Android...")
    subprocess.run([
        "am", "broadcast", "-a", "android.intent.action.MEDIA_SCANNER_SCAN_FILE",
        "-d", f"file://{carpeta_salida}"
    ], capture_output=True, text=True)
    speak("Proceso completado.")

def inicializar_spotify():
    try:
        return spotipy.Spotify(auth_manager=SpotifyClientCredentials(
            client_id=CLIENT_ID,
            client_secret=CLIENT_SECRET
        ))
        speak("Datos de Spotify leídos correctamente")
    except Exception as e:
        mensaje = f"Error al inicializar Spotify: {str(e)}"
        speak(mensaje)
        return None

def descargar_cancion(track_info, sp):
    try:
        tiempo_inicio = time.time()
        nombre = track_info['name']
        artista = track_info['artists'][0]['name']
        album = track_info['album']['name']
        portada_url = track_info['album']['images'][0]['url']
        
        query = f"{artista} {nombre} audio"
        
        ydl_opts = {
            'format': 'bestaudio/best',
            'quiet': True,
            'postprocessors': [{
                'key': 'FFmpegExtractAudio',
                'preferredcodec': '%s',
                'preferredquality': '%s',
            }],
            'outtmpl': f'/storage/emulated/0/Download/{artista} - {nombre}.%%(ext)s'
        }
        
        with YoutubeDL(ydl_opts) as ydl:
            speak(f"Descargando {nombre} de {artista}")
            info = ydl.extract_info(f"ytsearch:{query}", download=True)
        
        archivo = f'/storage/emulated/0/Download/{artista} - {nombre}.mp3'
        if os.path.exists(archivo):
            speak("Añadiendo metadatos")
            audio = ID3(archivo)
            audio.add(TIT2(encoding=3, text=nombre))
            audio.add(TPE1(encoding=3, text=artista))
            audio.add(TALB(encoding=3, text=album))
            portada = requests.get(portada_url).content
            audio.add(APIC(encoding=3, mime='image/jpeg', type=3, desc='Cover', data=portada))
            audio.save()
            actualizar_biblioteca()  # Actualizar la biblioteca al finalizar la descarga
            tiempo_final = time.time() - tiempo_inicio
            minutos = int(tiempo_final // 60)
            segundos = int(tiempo_final %% 60)
            mensaje = f"Descarga de {nombre} completada en {minutos}minutos {segundos}segundos"
            print(mensaje)
            speak(mensaje)
    except Exception as e:
        speak(f"Error al descargar {nombre}: {str(e)}")

def procesar_playlist(url_playlist, sp):
    try:
        speak("Obteniendo información de la playlist")
        playlist = sp.playlist_tracks(url_playlist)
        total_canciones = len(playlist['items'])
        speak(f"Se encontraron {total_canciones} canciones para descargar")
        for index, item in enumerate(playlist['items'], start=1):
            if item['track']:
                speak(f"Procesando canción {index} de {total_canciones}")
                descargar_cancion(item['track'], sp)
    except Exception as e:
        speak(f"Error al procesar playlist: {str(e)}")

if __name__ == "__main__":
    url = "%s"
    sp = inicializar_spotify()
    if not sp:
        speak("No se pudo inicializar Spotify. Verifica tus credenciales.")
        exit(1)
    
    try:
        if "playlist" in url:
            speak("Procesando playlist")
            procesar_playlist(url, sp)
        else:
            speak("Obteniendo información de la canción")
            track_info = sp.track(url)
            descargar_cancion(track_info, sp)
    except Exception as e:
        speak(f"Error: {str(e)}")
]]
          codigoPython = string.format(plantillaSpotify, selectedFormat, selectedQuality, URL)
        else
          if selectedType == "Audio" then
            local plantillaAudio = [[
#!/usr/bin/env python3
import yt_dlp
from gtts import gTTS
import time
import os
import tempfile
import subprocess

def speak(texto):
    """Función para dar retroalimentación por voz con gTTS acelerada"""
    try:
        with tempfile.NamedTemporaryFile(suffix='.mp3', delete=False) as temp_file:
            temp_path = temp_file.name
        
        tts = gTTS(text=texto, lang="es")
        tts.save(temp_path)
        
        # controlar la velocidad y el volumen del tts
        os.system(f"mpv --speed=1.3 --volume=100 {temp_path}")
        
        os.remove(temp_path)
    except Exception as e:
        print(f"Error en la síntesis de voz: {str(e)}")


def download_content(url):
    try:
        speak("Iniciando la descarga")
        tiempo_inicio = time.time()
        ydl_opts = {
            'format': 'bestaudio/best',
            'outtmpl': '/storage/emulated/0/Download/%%(title)s.%%(ext)s',
            'postprocessors': [{
                'key': 'FFmpegExtractAudio',
                'preferredcodec': '%s',
                'preferredquality': '%s',
            }],
            'noplaylist': %s,
            'ignoreerrors': True,
        }
        
        with yt_dlp.YoutubeDL(ydl_opts) as ydl:
            ydl.download([url])
            tiempo_final = time.time() - tiempo_inicio
            minutos = int(tiempo_final // 60)
            segundos = int(tiempo_final %% 60)
            mensaje = f"Descarga completada en {minutos}m {segundos}s"
            print(mensaje)
            speak(mensaje)
    except Exception as e:
        mensaje = f"Error durante la descarga: {str(e)}"
        print(mensaje)
        speak(mensaje)

if __name__ == "_main_":
    youtube_url = "%s"
    download_content(youtube_url)
    # Actualizar la base de datos multimedia de Android
    carpeta_salida = "/storage/emulated/0/Download/"
    speak("Actualizando la base de datos multimedia de Android...")
    subprocess.run([
        "am", "broadcast", "-a", "android.intent.action.MEDIA_SCANNER_SCAN_FILE",
        "-d", f"file://{carpeta_salida}"
    ], capture_output=True, text=True)
    speak("Proceso completado.")
]]
            codigoPython = string.format(plantillaAudio, selectedFormat, selectedQuality, not isPlaylist and "True" or "False", URL)
          else
            local plantillaVideo = [[
#!/usr/bin/env python3
import yt_dlp
import time
import os
import tempfile
from gtts import gTTS

def speak(texto):
    """Función para dar retroalimentación por voz con gTTS acelerada"""
    try:
        with tempfile.NamedTemporaryFile(suffix='.mp3', delete=False) as temp_file:
            temp_path = temp_file.name
        
        tts = gTTS(text=texto, lang="es")
        tts.save(temp_path)
        
        # controlar la velocidad y el volumen del tts
        os.system(f"mpv --speed=1.3 --volume=100 {temp_path}")
        
        os.remove(temp_path)
    except Exception as e:
        print(f"Error en la síntesis de voz: {str(e)}")

def download_content(url):
    try:
        speak("Iniciando la descarga")
        tiempo_inicio = time.time()
        ydl_opts = {
            'format': f'bestvideo[height<=%s][ext=%s]+bestaudio/best',
            'outtmpl': '/storage/emulated/0/Download/%%(title)s.%%(ext)s',
            'merge_output_format': '%s',
            'noplaylist': %s,
            'ignoreerrors': True,
        }
        
        with yt_dlp.YoutubeDL(ydl_opts) as ydl:
            ydl.download([url])
            tiempo_final = time.time() - tiempo_inicio
            minutos = int(tiempo_final // 60)
            segundos = int(tiempo_final %% 60)
            mensaje = f"Descarga completada en {minutos}m {segundos}s"
            print(mensaje)
            speak(mensaje)
    except Exception as e:
        mensaje = f"Error durante la descarga: {str(e)}"
        print(mensaje)
        speak(mensaje)

if __name__ == "__main__":
    youtube_url = "%s"
    download_content(youtube_url)
]]
            local height = selectedQuality:match("(%d+)p") or "1080"
            codigoPython = string.format(plantillaVideo, height, selectedFormat, selectedFormat, not isPlaylist and "True" or "False", URL)
          end
        end
        
        local rutaArchivo = "/storage/emulated/0/Documents/descargar_yt_dlp.py"
        local archivo = io.open(rutaArchivo, "w")
        if archivo then
          archivo:write(codigoPython)
          archivo:close()
          Toast.makeText(context, "Código Python generado en Documents", Toast.LENGTH_SHORT).show()
        else
          Toast.makeText(context, "Error al escribir en el archivo", Toast.LENGTH_SHORT).show()
        end
      end,
      layout_width = "match_parent",
      layout_height = "wrap_content",
      padding = "16dp",
    },
  }
  
  dlg = LuaDialog(service)
  local view = loadlayout(layout)
  dlg.View = view
  
  local function setSpinnerAdapter(spinner, items)
    local adapter = ArrayAdapter(context, android.R.layout.simple_spinner_item, items)
    adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
    spinner.setAdapter(adapter)
  end
  
  setSpinnerAdapter(SpinnerPlatform, plataformas)
  setSpinnerAdapter(SpinnerType, tipos)
  setSpinnerAdapter(SpinnerFormat, formatos_audio)
  setSpinnerAdapter(SpinnerQuality, calidades_audio)
  
  SpinnerType.onItemSelected = function(parent, view, position, id)
    local selectedType = tipos[position + 1]
    if selectedType == "Audio" then
      setSpinnerAdapter(SpinnerFormat, formatos_audio)
      setSpinnerAdapter(SpinnerQuality, calidades_audio)
    else
      setSpinnerAdapter(SpinnerFormat, formatos_video)
      setSpinnerAdapter(SpinnerQuality, calidades_video)
    end
  end
  
  dlg.show()
  vibrator.vibrate(100)
end

MostrarMenu()